/*
The "oslevent" event-handling library.  This is a library for
sequencing user-defined operations.

Orion Sky Lawlor, olawlor@acm.org, 2005/1/28 (Public Domain)
*/
/* (This "ifndef/define" stuff prevents problems if you include this header twice.) */
#ifndef __OSLEVENT_H 
#define __OSLEVENT_H

/**
  User-defined event class.  The purpose of this library is 
  to call the "execute" method of this class.
*/
class oslevent_class {
public:
	virtual ~oslevent_class(); /* compiler whines if you don't have a virtual destructor */
	
	/**
	  The user should override this to do whatever real
	  work is associated with the event: e.g., talk on 
	  the network, do some computation, etc.
	*/
	virtual void execute(void) =0;
};

/** Stores events inside an oslevent_pool.
  This is a "forward declaration"-- the real class is 
  defined inside oslevent.cpp.
*/
class oslevent_pool_queue; 

/**
  The library's main interface class: 
  an oslevent_pool stores a set of events for execution. 
*/
class oslevent_pool {
public:
	/**
	  Add this event to the pool.  It will run during
	  the next call to loop.
	*/
	void add(oslevent_class *evt);
	
	/**
	  Process all events in the pool until the pool is empty.
	*/
	void loop(void);
	
	oslevent_pool(); /* constructor: make an empty pool */
	~oslevent_pool(); /* destructor: clean out the pool */
private:
	oslevent_pool_queue *q;
};

#endif
